###################################################
### chunk number 1: 
###################################################
library(sampling)
ps.options(pointsize=12)
options(width=60)


###################################################
### chunk number 2: calib1
###################################################
data = rbind(matrix(rep("A", 150), 150, 1, byrow = TRUE), matrix(rep("B", 100), 100, 1, byrow = TRUE))
data = cbind.data.frame(data, c(rep(1, 60), rep(2,50), rep(3, 60), rep(1, 40), rep(2, 40)),
 1000 * runif(250))
sex = runif(nrow(data))
for (i in 1:length(sex)) if (sex[i] < 0.3) sex[i] = 1 else sex[i] = 2
data = cbind.data.frame(data, sex)
names(data) = c("state", "region", "income", "sex")
summary(data)



###################################################
### chunk number 3: calib2
###################################################
table(data$state)


###################################################
### chunk number 4: calib3
###################################################
s=strata(data,c("state"),size=c(25,10), method="srswor")


###################################################
### chunk number 5: calib31
###################################################
s=getdata(data,s)


###################################################
### chunk number 6: calib32
###################################################
status=runif(nrow(s))
for(i in 1:length(status))
 if(status[i]<0.3) status[i]=0 else status[i]=1
s=cbind.data.frame(s,status) 


###################################################
### chunk number 7: calib4
###################################################
s=rhg_strata(s,selection="region")


###################################################
### chunk number 8: calib5
###################################################
sr=s[s$status==1,]


###################################################
### chunk number 9: calib6
###################################################
X=matrix(0,nrow=nrow(data),ncol=5)
for(i in 1:nrow(data))
  {if(data$sex[i]==1) X[i,1]=1
   if(data$sex[i]==2) X[i,2]=1
   if(data$region[i]==1) X[i,3]=1
   if(data$region[i]==2) X[i,4]=1
   if(data$region[i]==3) X[i,5]=1
   }


###################################################
### chunk number 10: calib7
###################################################
total=c(t(rep(1,nrow(data)))%*%X)


###################################################
### chunk number 11: calib8
###################################################
Xs = X[sr$ID_unit,]
d = 1/(sr$Prob * sr$prob_resp)
summary(d)


###################################################
### chunk number 12: calib9
###################################################
g = calib(Xs, d, total, method = "linear")
summary(g)


###################################################
### chunk number 13: calib10
###################################################
w=d*g
summary(w)


###################################################
### chunk number 14: calib11
###################################################
checkcalibration(Xs, d, total, g)


###################################################
### chunk number 15: calib12
###################################################
cat("stratum 1\n")
data1=data[data$state=='A',]
X1=X[data$state=='A',]
total1=c(t(rep(1, nrow(data1))) %*% X1)
sr1=sr[sr$Stratum==1,]
Xs1=X[sr1$ID_unit,]
d1 = 1/(sr1$Prob * sr1$prob_resp)
g1=calib(Xs1, d1, total1, method = "linear")
checkcalibration(Xs1, d1, total1, g1)
cat("stratum 2\n")
data2=data[data$state=='B',]
X2=X[data$state=='B',]
total2=c(t(rep(1, nrow(data2))) %*% X2)
sr2=sr[sr$Stratum==2,]
Xs2=X[sr2$ID_unit,]
d2 = 1/(sr2$Prob * sr2$prob_resp)
g2=calib(Xs2, d2, total2, method = "linear")
checkcalibration(Xs2, d2, total2, g2)


###################################################
### chunk number 16:  eval=FALSE
###################################################
## data = rbind(matrix(rep("A", 150), 150, 1, byrow = TRUE), matrix(rep("B", 100), 100, 1, byrow = TRUE))
## data = cbind.data.frame(data, c(rep(1, 60), rep(2,50), rep(3, 60), rep(1, 40), rep(2, 40)),
##  1000 * runif(250))
## sex = runif(nrow(data))
## for (i in 1:length(sex)) if (sex[i] < 0.3) sex[i] = 1 else sex[i] = 2
## data = cbind.data.frame(data, sex)
## names(data) = c("state", "region", "income", "sex")
## summary(data)
## 
## table(data$state)
## s=strata(data,c("state"),size=c(25,10), method="srswor")
## s=getdata(data,s)
## status=runif(nrow(s))
## for(i in 1:length(status))
##  if(status[i]<0.3) status[i]=0 else status[i]=1
## s=cbind.data.frame(s,status) 
## s=rhg_strata(s,selection="region")
## sr=s[s$status==1,]
## X=matrix(0,nrow=nrow(data),ncol=5)
## for(i in 1:nrow(data))
##   {if(data$sex[i]==1) X[i,1]=1
##    if(data$sex[i]==2) X[i,2]=1
##    if(data$region[i]==1) X[i,3]=1
##    if(data$region[i]==2) X[i,4]=1
##    if(data$region[i]==3) X[i,5]=1
##    }
## total=c(t(rep(1,nrow(data)))%*%X)
## Xs = X[sr$ID_unit,]
## d = 1/(sr$Prob * sr$prob_resp)
## summary(d)
## g = calib(Xs, d, total, method = "linear")
## summary(g)
## w=d*g
## summary(w)
## checkcalibration(Xs, d, total, g)
## cat("stratum 1\n")
## data1=data[data$state=='A',]
## X1=X[data$state=='A',]
## total1=c(t(rep(1, nrow(data1))) %*% X1)
## sr1=sr[sr$Stratum==1,]
## Xs1=X[sr1$ID_unit,]
## d1 = 1/(sr1$Prob * sr1$prob_resp)
## g1=calib(Xs1, d1, total1, method = "linear")
## checkcalibration(Xs1, d1, total1, g1)
## cat("stratum 2\n")
## data2=data[data$state=='B',]
## X2=X[data$state=='B',]
## total2=c(t(rep(1, nrow(data2))) %*% X2)
## sr2=sr[sr$Stratum==2,]
## Xs2=X[sr2$ID_unit,]
## d2 = 1/(sr2$Prob * sr2$prob_resp)
## g2=calib(Xs2, d2, total2, method = "linear")
## checkcalibration(Xs2, d2, total2, g2)
## 
## 
## 
## sampling.newpage()
## 


###################################################
### chunk number 17: ex1
###################################################
X=cbind(c(rep(1,50),rep(0,50)),c(rep(0,50),rep(1,50)),1:100)
# vector of population totals
total=apply(X,2,"sum")
Z=150:249 
# the variable of interest
Y=5*Z*(rnorm(100,0,sqrt(1/3))+apply(X,1,"sum"))
# inclusion probabilities
pik=inclusionprobabilities(Z,20)
# joint inclusion probabilities 
pikl=UPtillepi2(pik)
# number of simulations; let nsim=10000 for an accurate result
nsim=10
c1=c2=c3=c4=c5=numeric(nsim)
for(i in 1:nsim)
{
# draws a sample
s=UPtille(pik)
# computes the inclusion prob. for the sample
piks=pik[s==1]
# the sample matrix of auxiliary information
Xs=X[s==1,]
# computes the g-weights
g=calib(Xs,d=1/piks,total,method="linear")
# computes the variable of interest in the sample
Ys=Y[s==1]
# computes the joint inclusion prob. for the sample
pikls=pikl[s==1,s==1]
# computes the calibration estimator and its variance estimation
cc=calibev(Ys,Xs,total,pikls,d=1/piks,g,with=FALSE,EPS=1e-6)
c1[i]=cc$calest
c2[i]=cc$evar
# computes the variance estimator of the calibration estimator (second method)
c3[i]=varest(Ys,Xs,pik=piks,w=g/piks)
# computes the variance estimator of the the Horvitz-Thompson estimator
c4[i]=varest(Ys=Ys,pik=piks) 
# computes the Horvitz-Thompson estimator
c5[i]=HTestimator(Ys,piks)
}
cat("the population total:",sum(Y),"\n")
cat("the calibration estimator under simulations:", mean(c1),"\n")

N=length(Y)
delta=matrix(0,N,N)
for(k in 1:(N-1))
  for(l in (k+1):N)
   delta[k,l]=delta[l,k]=pikl[k,l]-pik[k]*pik[l]
diag(delta)=pik*(1-pik)
varHT=0
varasym=0
e=lm(Y~X)$resid
for(k in 1:N)
  for(l in 1:N) {varHT=varHT+Y[k]*Y[l]*delta[k,l]/(pik[k]*pik[l])
                 varasym=varasym+e[k]*e[l]*delta[k,l]/(pik[k]*pik[l])} 
cat("the approximate variance of the calibration estimator:",varasym,"\n")
cat("first variance estimator of the calibration est. using calibev function:\n")
cat("MSE of the first variance estimator:", var(c2)+(mean(c2)-varasym)^2,"\n")
cat("second variance estimator of the calibration est. using varest function:\n")
cat("MSE of the second variance estimator:", var(c3)+(mean(c3)-varasym)^2,"\n")
cat("the Horvitz-Thompson estimator under simulations:", mean(c5),"\n")
cat("the variance of the Horvitz-Thompson estimator:", varHT, "\n")
cat("the variance estimator of the H-T estimator under simulations:", mean(c4),"\n")
cat("MSE of the variance estimator:", var(c4)+(mean(c4)-varHT)^2,"\n")


###################################################
### chunk number 18:  eval=FALSE
###################################################
## X=cbind(c(rep(1,50),rep(0,50)),c(rep(0,50),rep(1,50)),1:100)
## # vector of population totals
## total=apply(X,2,"sum")
## Z=150:249 
## # the variable of interest
## Y=5*Z*(rnorm(100,0,sqrt(1/3))+apply(X,1,"sum"))
## # inclusion probabilities
## pik=inclusionprobabilities(Z,20)
## # joint inclusion probabilities 
## pikl=UPtillepi2(pik)
## # number of simulations; let nsim=10000 for an accurate result
## nsim=10
## c1=c2=c3=c4=c5=numeric(nsim)
## for(i in 1:nsim)
## {
## # draws a sample
## s=UPtille(pik)
## # computes the inclusion prob. for the sample
## piks=pik[s==1]
## # the sample matrix of auxiliary information
## Xs=X[s==1,]
## # computes the g-weights
## g=calib(Xs,d=1/piks,total,method="linear")
## # computes the variable of interest in the sample
## Ys=Y[s==1]
## # computes the joint inclusion prob. for the sample
## pikls=pikl[s==1,s==1]
## # computes the calibration estimator and its variance estimation
## cc=calibev(Ys,Xs,total,pikls,d=1/piks,g,with=FALSE,EPS=1e-6)
## c1[i]=cc$calest
## c2[i]=cc$evar
## # computes the variance estimator of the calibration estimator (second method)
## c3[i]=varest(Ys,Xs,pik=piks,w=g/piks)
## # computes the variance estimator of the the Horvitz-Thompson estimator
## c4[i]=varest(Ys=Ys,pik=piks) 
## # computes the Horvitz-Thompson estimator
## c5[i]=HTestimator(Ys,piks)
## }
## cat("the population total:",sum(Y),"\n")
## cat("the calibration estimator under simulations:", mean(c1),"\n")
## 
## N=length(Y)
## delta=matrix(0,N,N)
## for(k in 1:(N-1))
##   for(l in (k+1):N)
##    delta[k,l]=delta[l,k]=pikl[k,l]-pik[k]*pik[l]
## diag(delta)=pik*(1-pik)
## varHT=0
## varasym=0
## e=lm(Y~X)$resid
## for(k in 1:N)
##   for(l in 1:N) {varHT=varHT+Y[k]*Y[l]*delta[k,l]/(pik[k]*pik[l])
##                  varasym=varasym+e[k]*e[l]*delta[k,l]/(pik[k]*pik[l])} 
## cat("the approximate variance of the calibration estimator:",varasym,"\n")
## cat("first variance estimator of the calibration est. using calibev function:\n")
## cat("MSE of the first variance estimator:", var(c2)+(mean(c2)-varasym)^2,"\n")
## cat("second variance estimator of the calibration est. using varest function:\n")
## cat("MSE of the second variance estimator:", var(c3)+(mean(c3)-varasym)^2,"\n")
## cat("the Horvitz-Thompson estimator under simulations:", mean(c5),"\n")
## cat("the variance of the Horvitz-Thompson estimator:", varHT, "\n")
## cat("the variance estimator of the H-T estimator under simulations:", mean(c4),"\n")
## cat("MSE of the variance estimator:", var(c4)+(mean(c4)-varHT)^2,"\n")
## 
## sampling.newpage()
## 


